@@ -415,7 +415,6 @@ class table__g_l_y_f(DefaultTable.DefaultTable):
             controls = []
 
             for component in glyph.components:
-
                 (
                     componentCoords,
                     componentControls,
@@ -1424,6 +1423,7 @@ class Glyph(object):
             coordinates = coordinates.copy()
             coordinates.translate((offset, 0))
         start = 0
+        maybeInt = lambda v: int(v) if v == int(v) else v
         for end in endPts:
             end = end + 1
             contour = coordinates[start:end]
@@ -1436,11 +1436,18 @@ class Glyph(object):
                 if cubic:
                     count = len(contour)
                     assert count % 2 == 0, "Odd number of cubic off-curves undefined"
+                    l = contour[-1]
+                    f = contour[0]
+                    p0 = (maybeInt((l[0] + f[0]) * 0.5), maybeInt((l[1] + f[1]) * 0.5))
+                    pen.moveTo(p0)
                     for i in range(0, count, 2):
                         p1 = contour[i]
                         p2 = contour[i + 1]
                         p4 = contour[i + 2 if i + 2 < count else 0]
-                        p3 = ((p2[0] + p4[0]) * 0.5, (p2[1] + p4[1]) * 0.5)
+                        p3 = (
+                            maybeInt((p2[0] + p4[0]) * 0.5),
+                            maybeInt((p2[1] + p4[1]) * 0.5),
+                        )
                         pen.curveTo(p1, p2, p3)
                 else:
                     # There is not a single on-curve point on the curve,
@@ -1484,7 +1491,10 @@ class Glyph(object):
                                 p1 = contour[i]
                                 p2 = contour[i + 1]
                                 p4 = contour[i + 2]
-                                p3 = ((p2[0] + p4[0]) * 0.5, (p2[1] + p4[1]) * 0.5)
+                                p3 = (
+                                    maybeInt((p2[0] + p4[0]) * 0.5),
+                                    maybeInt((p2[1] + p4[1]) * 0.5),
+                                )
                                 lastOnCurve = p3
                                 pen.curveTo(p1, p2, p3)
                             pen.curveTo(*contour[count - 3 : count])
@@ -1518,9 +1528,15 @@ class Glyph(object):
             start = end
             pen.beginPath()
             # Start with the appropriate segment type based on the final segment
-            segmentType = "line" if cFlags[-1] == 1 else "qcurve"
+
+            if cFlags[-1] & flagOnCurve:
+                segmentType = "line"
+            elif cFlags[-1] & flagCubic:
+                segmentType = "curve"
+            else:
+                segmentType = "qcurve"
             for i, pt in enumerate(contour):
-                if cFlags[i] & flagOnCurve == 1:
+                if cFlags[i] & flagOnCurve:
                     pen.addPoint(pt, segmentType=segmentType)
                     segmentType = "line"
                 else:
@@ -1925,7 +1941,6 @@ VAR_COMPONENT_TRANSFORM_MAPPING = {
 
 
 class GlyphVarComponent(object):
-
     MIN_SIZE = 5
 
     def __init__(self):
@@ -2161,7 +2176,6 @@ class GlyphVarComponent(object):
         return count
 
     def getCoordinatesAndControls(self):
-
         coords = []
         controls = []
 
